from lopt.models import Language, ModelT


class LoptError(Exception):
    def __init__(self, msg: str) -> None:
        super().__init__(msg)
        self.msg: str = msg

    def __str__(self) -> str:
        return self.msg


class NoCodeFoundError(LoptError):
    def __init__(
        self,
        msg: str = "No code found",
        *,
        lang: Language,
        data: str,
    ) -> None:
        super().__init__(msg)
        self.lang: Language = lang
        self.data: str = data

    def __str__(self) -> str:
        return f"{self.msg}, lang: {self.lang}, data: {self.data}"


class ParseObjectError(LoptError):
    def __init__(
        self,
        msg: str = "Failed to parse model",
        *,
        data: str,
    ) -> None:
        super().__init__(msg)
        self.data: str = data

    def __str__(self) -> str:
        return f"{self.msg}, data: {self.data}"


class ParseModelError(LoptError):
    def __init__(
        self,
        msg: str = "Failed to parse model",
        *,
        model: type[ModelT],
        data: str,
    ) -> None:
        super().__init__(msg)
        self.model: type = model
        self.data: str = data

    def __str__(self) -> str:
        return f"{self.msg}, model: {self.model.__name__}, data: {self.data}"



import json
import re
from typing import Any

import yaml
from pydantic import ValidationError

from lopt.errors import NoCodeFoundError, ParseModelError, ParseObjectError
from lopt.models import Language, ModelT


def parse_multi_line(s: str, *, lang: Language) -> str | None:
    if match := re.search(rf"```(?:{re.escape(lang)})?(.+)```", s, re.DOTALL):
        return match[1]

    return None


def parse_inline(s: str) -> str | None:
    if match := re.search(r"`(.+)`", s):
        return match[1]

    return None


def parse_code(s: str, *, lang: Language) -> str:
    if multi_line := parse_multi_line(s, lang=lang):
        return multi_line

    if inline := parse_inline(s):
        return inline

    raise NoCodeFoundError(data=s, lang=lang)


def _parse_object(
    s: str,
    *,
    lang: Language,
    ignore_errors: bool = True,
) -> Any:
    parsers = {
        "json": json.loads,
        "yaml": yaml.safe_load,
    }

    try:
        return parsers[lang](s)

    except BaseException as error:
        if not ignore_errors:
            raise ParseObjectError(data=s) from error

    return None


def parse_object(
    s: str,
    *,
    lang: Language,
    ignore_errors: bool = True,
) -> Any | None:
    try:
        data_str = parse_code(s, lang=lang)

    except NoCodeFoundError:
        if not ignore_errors:
            raise

        data_str = s.strip("`")

    if not data_str:
        if ignore_errors:
            return None

        raise NoCodeFoundError(data=s, lang=lang)

    return _parse_object(data_str, lang=lang, ignore_errors=ignore_errors)


def parse_model(
    s: str,
    *,
    lang: Language,
    model: type[ModelT],
    ignore_errors: bool = True,
) -> ModelT | None:
    try:
        data_str = parse_code(s, lang=lang)

    except NoCodeFoundError:
        data_str = s.strip("`")

    if not data_str:
        if ignore_errors:
            return None

        raise NoCodeFoundError(data=s, lang=lang)

    try:
        obj = _parse_object(data_str, lang=lang)

    except ParseObjectError:
        if not ignore_errors:
            raise

        return None

    else:
        try:
            return model.model_validate(obj)

        except ValidationError as error:
            if not ignore_errors:
                raise ParseModelError(model=model, data=data_str) from error

            return None



import json as jsonlib
from typing import Any

import yaml as yamllib


def unordered_list(items: list[Any], *, bullet: str = "-") -> str:
    return "\n".join(f"{bullet} {item}" for item in items)


def ordered_list(items: list[Any], *, zero_indexed: bool = False) -> str:
    offset = not zero_indexed

    return "\n".join(f"{i + offset}. {item}" for i, item in enumerate(items))


def json(obj: Any, **dump_options: Any) -> str:
    return jsonlib.dumps(obj, **dump_options)


def yaml(obj: Any, **dump_options: Any) -> str:
    return yamllib.dump(obj, **dump_options)  # pyright: ignore[reportUnknownVariableType]



from lopt import formatter, parser
from lopt.errors import LoptError, NoCodeFoundError, ParseModelError, ParseObjectError
from lopt.parser import (
    parse_code,
    parse_inline,
    parse_model,
    parse_multi_line,
    parse_object,
)

__all__ = [
    "LoptError",
    "NoCodeFoundError",
    "ParseModelError",
    "ParseObjectError",
    "formatter",
    "parse_code",
    "parse_inline",
    "parse_model",
    "parse_multi_line",
    "parse_object",
    "parser",
]
